use ::static_files::resource::new_resource as n;
use ::std::include_bytes as i;
use ::std::collections::HashMap;
use ::static_files::Resource;
mod set_1;
#[allow(clippy::unreadable_literal)] #[must_use] pub fn generate() -> ::std::collections::HashMap<&'static str, ::static_files::Resource> {
let mut r = ::std::collections::HashMap::new();
set_1::generate(&mut r);
r
}
